"""
Module that contains the command line app.

Why does this file exist, and why not put this in __main__?

  You might be tempted to import things from __main__ later, but that will cause
  problems: the code will get executed twice:

  - When you run `python -msnyk_test` python will execute
    ``__main__.py`` as a script. That means there won't be any
    ``snyk_test.__main__`` in ``sys.modules``.
  - When you import __main__ it will get executed again (as a module) because
    there's no ``snyk_test.__main__`` in ``sys.modules``.

  Also see (1) from http://click.pocoo.org/5/setuptools/#setuptools-integration
"""
import argparse
import json
from sys import argv

from pandas import json_normalize

from snyk_test.Package import Package


def parse_args(args):
    parser = argparse.ArgumentParser(description='Process optional arguments.')
    parser.add_argument('package', help='Specify the package name and version')
    parser.add_argument('--json', dest="is_json", action='store_true',
                        help='Format the result as JSON (default: false)')

    return parser.parse_args(args)


def main(argv=argv):
    """
    Args:
        argv: arguments list from command line

    Returns:
        int: A return code

    """

    parser = parse_args(argv[1:])

    package = Package(parser.package.split('@')[0], parser.package.split('@')[1])
    package.run_assesment()

    if parser.is_json:
        print(
            json.dumps({
                "security_vulnerabilities": package.vulnerabilities, "license_issues": package.license_issues
            })
        )
    else:
        print("Security Vulnerabilities:")
        print(json_normalize(package.vulnerabilities))
        print("\n")
        print("License Issues:")
        print(json_normalize(package.license_issues))
        print("\n")

    return 0



from snyk import SnykClient

from snyk_test.utils import get_default_token_path
from snyk_test.utils import get_org_id
from snyk_test.utils import get_token


class Package:
    def __init__(self, name, version):
        self.name = name
        self.version = version
        self.vulnerabilities = []
        self.license_issues = []

    def run_assesment(self):
        snyk_token_path = get_default_token_path()
        snyk_token = get_token(snyk_token_path)
        org_id = get_org_id(snyk_token_path)

        client = SnykClient(snyk_token)
        org_client = client.organizations.get(org_id)

        print("Testing package %s@%s" % (self.name, self.version))
        print("=====================")

        snyk_result = org_client.test_python(self.name, self.version)

        vulnerabilities_output = snyk_result.issues.vulnerabilities
        license_issues_output = snyk_result.issues.licenses

        severity_order = ['critical', 'high', 'medium', 'low']

        if vulnerabilities_output is not None:
            for v in vulnerabilities_output:
                formatted_vulnerability = {
                    'id': v.id,
                    'title': v.title,
                    'url': v.url,
                    'package': v.package,
                    'version': v.version,
                    'identifiers': v.identifiers['CVE'],
                    'severity': v.severity,
                    'language': v.language,
                    'package_manager': v.packageManager,
                    'is_upgradable': v.isUpgradable,
                    'is_patchable': v.isPatchable
                }

                self.vulnerabilities.append(formatted_vulnerability)
            self.vulnerabilities = sorted(self.vulnerabilities, key=lambda k: severity_order.index(k['severity']))

        if license_issues_output is not None:
            for i in license_issues_output:
                formatted_license_issue = {
                    'id': i.id,
                    'title': i.title,
                    'url': i.url,
                    'package': i.package,
                    'version': i.version,
                    'severity': i.severity,
                    'is_ignored': i.isIgnored,
                    'is_patched': i.isPatched,
                    'language': i.language,
                    'priority_score': i.priorityScore,
                    'package_manager': i.packageManager
                }

                self.license_issues.append(formatted_license_issue)
            self.license_issues = sorted(self.license_issues, key=lambda k: severity_order.index(k['severity']))



from snyk.models import Issue
from snyk.models import IssueSet
from snyk.models import Vulnerability

from snyk_test.cli import main


def test_main(mocker):
    mocker.patch('snyk_test.Package.Package.run_assesment',
                 return_value=IssueSet(ok=False,
                                       packageManager='pip',
                                       dependencyCount=27,
                                       issues=Issue(
                                           vulnerabilities=[
                                               Vulnerability(
                                                   id='SNYK-PYTHON-CHANNELSREDIS-574718',
                                                   url='https://snyk.io/vuln/SNYK-PYTHON-CHANNELSREDIS-574718',
                                                   title='Denial of Service (DoS)',
                                                   description='## Overview\n[channels-redis]('
                                                               'https://www.pypi.org/project/channels-redis) is '
                                                               'a Redis-backed ASGI channel layer '
                                                               'implementation\n\nAffected versions of this '
                                                               'package are vulnerable to Denial of Service ('
                                                               'DoS) due to incorrect expiration '
                                                               'mechanisms.\n\n## Details\n\nDenial of Service '
                                                               '(DoS) describes a family of attacks, '
                                                               'all aimed at making a system inaccessible to '
                                                               'its intended and legitimate users.\n\nUnlike '
                                                               'other vulnerabilities, DoS attacks usually do '
                                                               'not aim at breaching security. Rather, '
                                                               'they are focused on making websites and '
                                                               'services unavailable to genuine users resulting '
                                                               'in downtime.\n\nOne popular Denial of Service '
                                                               'vulnerability is DDoS (a Distributed Denial of '
                                                               'Service), an attack that attempts to clog '
                                                               'network pipes to the system by generating a '
                                                               'large volume of traffic from many '
                                                               'machines.\n\nWhen it comes to open source '
                                                               'libraries, DoS vulnerabilities allow attackers '
                                                               'to trigger such a crash or crippling of the '
                                                               'service by using a flaw either in the '
                                                               'application code or from the use of open source '
                                                               'libraries.\n\nTwo common types of DoS '
                                                               'vulnerabilities:\n\n* High CPU/Memory '
                                                               'Consumption- An attacker sending crafted '
                                                               'requests that could cause the system to take a '
                                                               'disproportionate amount of time to process. For '
                                                               'example, '
                                                               '[commons-fileupload:commons-fileupload]('
                                                               'SNYK-JAVA-COMMONSFILEUPLOAD-30082).\n\n* Crash '
                                                               '- An attacker sending crafted requests that '
                                                               'could cause the system to crash. For Example,  '
                                                               '[npm `ws` package](npm:ws:20171108)\n\n## '
                                                               'Remediation\nUpgrade `channels-redis` to '
                                                               'version 3.0.0 or higher.\n## References\n- ['
                                                               'GitHub Commit]('
                                                               'https://github.com/django/channels_redis/commit'
                                                               '/c13b3453f976e23a617fc55f9748609c1692c8f0)\n- ['
                                                               'GitHub Issue]('
                                                               'https://github.com/django/channels_redis/issues'
                                                               '/182)\n- [GitHub PR]('
                                                               'https://github.com/django/channels_redis/pull'
                                                               '/184)\n',
                                                   upgradePath=[],
                                                   package='channels-redis',
                                                   version='2.4.2',
                                                   severity='medium',
                                                   exploitMaturity='no-known-exploit',
                                                   isUpgradable=False,
                                                   isPatchable=False,
                                                   identifiers={
                                                       'CVE': [],
                                                       'CWE': [
                                                           'CWE-400']},
                                                   semver={
                                                       'vulnerable': [
                                                           '[0,3.0.0)']},
                                                   fromPackages=[],
                                                   language='python',
                                                   packageManager='pip',
                                                   publicationTime='2020-07-16T13:58:02.513987Z',
                                                   disclosureTime='2020-06-28T14:12:54Z',
                                                   credit=[
                                                       'astutejoe'],
                                                   CVSSv3='CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:N/I:N/A:L',
                                                   cvssScore=5.3,
                                                   ignored=[],
                                                   patched=[])],
                                           licenses=[])))

    testargs = ["snyk-test", "channels-redis@2.4.2"]
    mocker.patch('sys.argv', testargs)
    assert main(argv=testargs) == 0

    testargs = ["snyk-test", "channels-redis@2.4.2", "--json"]
    mocker.patch('sys.argv', testargs)
    assert main(argv=testargs) == 0



import json
from pathlib import Path


def get_default_token_path():
    home = str(Path.home())
    default_token_path = "%s/.config/configstore/snyk.json" % home
    return default_token_path


def get_token(token_file_path):
    path = token_file_path

    try:
        with open(path, "r") as f:
            json_obj = json.load(f)
            token = json_obj["api"]
            return token
    except FileNotFoundError as fnfe:
        print("Snyk auth token not found at %s" % path)
        print("Run `snyk auth` (see https://github.com/snyk/snyk#installation) or manually create this file with your token.")
        raise fnfe
    except KeyError as ke:
        print("Snyk auth token file is not properly formed: %s" % path)
        print("Run `snyk auth` (see https://github.com/snyk/snyk#installation) or manually create this file with your token.")
        raise ke


def get_org_id(token_file_path):
    path = token_file_path

    try:
        with open(path, "r") as f:
            json_obj = json.load(f)
            org_id = json_obj["org_id"]
            return org_id
    except FileNotFoundError as fnfe:
        print("Snyk org_id not found at %s" % path)
        print("Run `snyk auth` (see https://github.com/snyk/snyk#installation) or manually create this file with your "
              "token, then manually add the org id.")
        raise fnfe
    except KeyError as ke:
        print("Snyk auth token file is not properly formed: %s" % path)
        print("Run `snyk auth` (see https://github.com/snyk/snyk#installation) or manually create this file with your token.")
        raise ke
